function printpf(bus, gen, branch, success, et, fd, pretty, ugly)
%PRINTPF   Prints power flow results.
%   printpf(bus, gen, branch, success, et, fd, pretty, ugly) prints power flow
%   results to fd (a file descriptor). If pretty is true it formats things
%   nicely with headings and stuff. If ugly is true it prints some raw
%   data for our Perl database interface code.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialization -----
%% default arguments
if nargin < 8
	ugly = 0;			%% don't print the raw stuff
	if nargin < 7
		pretty = 1;		%% print the pretty stuff
		if nargin < 6
			fd = 1;		%% print to stdio by default
		end
	end
end

%% define named indices into bus, gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% parameters
nb = size(bus, 1);
nl = size(branch, 1);
ng = size(gen, 1);

%%----- print the stuff -----
if success
	if pretty
		%% elapsed time
		fprintf(fd, '\nConverged in %.2f seconds', et);
		
		%% bus data
		fprintf(fd, '\n===========================  Bus Data  ===========================');
		fprintf(fd, '\nBus         Voltage            Generation             Load');
		fprintf(fd, '\n #    Mag (pu)   Ang (deg)   P (MW)   Q (MVAR)   P (MW)   Q (MVAR)');
		fprintf(fd, '\n---  ----------  ---------  --------  --------  --------  --------');
		for i = 1:nb
			fprintf(fd, '\n%3d%10.4f%11.3f', bus(i, [BUS_I, VM, VA]));
			g = find(gen(:, GEN_BUS) == bus(i, BUS_I));
			if gen(g, GEN_STATUS)
				fprintf(fd, '%11.2f%10.2f', gen(g, [PG, QG]));
			else
				fprintf(fd, '        -         -  ');
			end
			if bus(i, PD) | bus(i, QD)
				fprintf(fd, '%10.2f%10.2f', bus(i, [PD, QD]));
			else
				fprintf(fd, '       -         -  ');
			end
		end
		fprintf(fd, '\n');
		
		%% branch data
		fprintf(fd, '\n================  Branch Data  =====================');
		fprintf(fd, '\n From   To        Avg. Flow             Loss');
		fprintf(fd, '\n Bus    Bus    P (MW)   Q (MVAR)   P (MW)   Q (MVAR)');
		fprintf(fd, '\n-----  -----  --------  --------  --------  --------');
		fprintf(fd, '\n%4d%7d%10.2f%10.2f%10.3f%10.2f', ...
				[	branch(:, [F_BUS, T_BUS]), ...
					(branch(:, [PF, QF]) - branch(:, [PT, QT]))/2, ...
					branch(:, [PF, QF]) + branch(:, [PT, QT]) ...
				]');
		fprintf(fd, '\n');
	end
	
	%% print raw data for Perl database interface
	if ugly
		fprintf(fd, '----------  raw PS::Soln data below  ----------\n');
		fprintf(fd, 'bus\n');
		fprintf(fd, '%d\t%d\t%f\t%f\t%d\t%d\t%d\t%d\n', ...
					[bus(:, [BUS_I, BUS_TYPE, VM, VA]) zeros(nb, 4)]');
	
		fprintf(fd, 'branch\n');
		fprintf(fd, '%d\t%f\t%f\t%f\t%f\t%d\t%d\n', ...
					[[1:nl]' branch(:, [PF, QF, PT, QT]) zeros(nl, 2)]');
	
		fprintf(fd, 'gen\n');
		fprintf(fd, '%d\t%f\t%f\t%f\t%d\t%d\t%d\t%d\t%d\n', ...
					[[1:ng]' gen(:, [PG, QG, VG, GEN_STATUS]) zeros(ng, 4)]');
		fprintf(fd, 'end\n');
		fprintf(fd, '----------  raw PS::Soln data above  ----------\n');
	end
else
	fprintf(fd, 'did not converge\n');
end

return;
